
import 'package:dio/dio.dart';

import 'log_interceptor.dart';

/// dio的封装
class HttpUtil {

  String _codeField = "code";
  String _dataField = "data";
  String _msgField = "msg";
  dynamic _successCode = "200";

  HttpUtil._private();

  static HttpUtil _instance;
  static Dio _dio;

  factory HttpUtil() {
    if (_instance == null) {
      _instance = HttpUtil._private();
      _dio = Dio();
      _dio.options.connectTimeout = 5000; //5s
      _dio.options.receiveTimeout = 3000;
      _dio.interceptors
          .add(BetterLogInterceptor(responseBody: true, requestBody: true));
    }
    return _instance;
  }

  String _baseUrl;

  String getBaseUrl() => _baseUrl;

  HttpUtil baseUrl(String baseUrl) {
    _baseUrl = baseUrl;
    _dio.options.baseUrl = baseUrl;
    return this;
  }

  HttpUtil configCodeMsg({String codeField, String dataField, String msgField, String successCode}){
    _codeField = codeField;
    _dataField = dataField;
    _msgField = msgField;
    _successCode = successCode;
    return this;
  }

  HttpUtil addHeader(Map<String, dynamic> header) {
    _dio.options.headers.addAll(header);
    return this;
  }

  HttpUtil addInterceptor(InterceptorsWrapper interceptor) {
    _dio.interceptors.insert(0, interceptor);
    return this;
  }

  Future get(String url) async => _getData((await _dio.get(url)).data);

  /// 默认是json编码
  Future post(String url, Map<String, dynamic> data) async =>
      _getData((await _dio.post(url, data: data)).data);

  Future<Map<String, dynamic>> postForm(
      String url, Map<String, dynamic> map) async =>
      _getData((await _dio.post(url, data: FormData.fromMap(map))).data);


  Future<Map<String, dynamic>> put(
      String url, Map<String, dynamic> data) async =>
    _getData((await _dio.put(url, data: data)).data);


  Future<Map<String, dynamic>> delete(
      String url, Map<String, dynamic> data) async =>
    _getData((await _dio.delete(url, data: data)).data);

  Future<ResponseBody> download(String url,
      {ProgressCallback onReceiveProgress}) async {
    return (await _dio.get<ResponseBody>(url,
            options: RequestOptions(responseType: ResponseType.stream),
            onReceiveProgress: onReceiveProgress))
        .data;
  }

  bool _isSuccess(Map<String, dynamic> map) => map[_codeField] == _successCode;

  dynamic _getData(Map<String, dynamic> res) {
    if (_isSuccess(res)) return res[_dataField];
    return null;
  }
}
