import 'package:flutter/material.dart';

enum IconPosition { Left, Top, Right, Bottom }

/// 同时设置图片和文字
class SuperText extends StatelessWidget {
  final IconPosition iconPosition;
  final Image image;
  final double iconSpace;
  final String text;
  final TextStyle textStyle;
  final double width;
  final double height;
  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry margin;
  final Color bgColor;
  final AlignmentGeometry alignment;
  final BorderRadiusGeometry borderRadius;
  final Gradient gradient;
  final BoxBorder border;
  final VoidCallback onTap;
  final BoxConstraints constraints;
  final bool hasInkWell;

  SuperText(
      {this.iconPosition = IconPosition.Left,
      this.image,
      this.iconSpace = 0,
      this.text,
      this.textStyle,
      this.width,
      this.height,
      this.padding,
      this.margin,
      this.bgColor,
      this.alignment = Alignment.center,
      this.borderRadius,
      this.gradient,
      this.border,
      this.onTap,
      this.hasInkWell = false,
      this.constraints});

  @override
  Widget build(BuildContext context) {
    return onTap == null
        ? buildContainer((this.image == null ? buildChild() : buildDirection()))
        : (hasInkWell
            ? InkWell(
                onTap: onTap,
                child: buildContainer(
                    this.image == null ? buildChild() : buildDirection()))
            : GestureDetector(
                behavior: HitTestBehavior.translucent,
                onTap: onTap,
                child: buildContainer(
                    this.image == null ? buildChild() : buildDirection())));
  }

  Widget buildContainer(Widget child) {
    return Container(
      width: width,
      height: height,
      padding: padding,
      margin: margin,
      alignment: alignment,
      constraints: constraints,
      decoration: BoxDecoration(
          color: bgColor,
          borderRadius: borderRadius,
          gradient: gradient,
          border: border),
      child: child,
    );
  }

  Widget buildDirection() {
    if (iconPosition == IconPosition.Left ||
        iconPosition == IconPosition.Right) {
      return Row(
          children: buildChildren(),
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.center);
    }
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: buildChildren(),
    );
  }

  Widget buildChild() {
    return Text(
      text,
      style: textStyle,
    );
  }

  List<Widget> buildChildren() {
    if (iconPosition == IconPosition.Left || iconPosition == IconPosition.Top) {
      return <Widget>[
        if (image != null) ...<Widget>[image, buildSizeBox()],
        Text(
          text,
          style: textStyle,
        )
      ];
    } else {
      return <Widget>[
        Text(
          text,
          style: textStyle,
        ),
        if (image != null) ...<Widget>[buildSizeBox(), image]
      ];
    }
  }

  Widget buildSizeBox() {
    if (iconPosition == IconPosition.Left ||
        iconPosition == IconPosition.Right) {
      return SizedBox(
        width: iconSpace,
      );
    } else {
      return SizedBox(
        height: iconSpace,
      );
    }
  }
}
